#!/bin/zsh

auth=""
server_dns=""

# DO NOT EDIT BELOW THIS LINE

if [[ "$auth" == "" ]]
then
	echo "\$auth variable not set.  Edit script to add an API base 64 token as a value for this variable"
	exit 1
fi

if [[ "$server_dns" == "" ]]
then
	echo "\$server_dns variable not set.  Edit script to add a value for the variable.  Use the Server Name from the Mobiles tab view"
	exit 1
fi

# Requires two Custom Fields with the following internal names:
# macos_mdm_system_date
# macos_mdm_user_date
# Both should be date formatted and will hold MDM response date commands
# If no date is found when checking device's command history an old date will be populated

# STATUS
# https://confluence.filewave.com/display/mdm/Device+Management

# STATUS_CHOICES = (
#     (0, 'NotSent'),
#     (1, 'Acknowledged'),
#     (2, 'Error'),
#     (3, 'CommandFormatError'),
#     (4, 'NotNow'),
#     (5, 'Sent')
# )
# 'not sent' messages are not included in ios_commands.  They need to progess to another status to be added to this table.

# We should have a DeviceInformation command for each user and the system user.
# Multiple entries can exist for any one command per user, but there should only be one per status type.  Targeting 'acknowledged'
# Common for a 'not sent' and an 'acknowledged' to coexist where new commands have been queued.


# Device list to fill responses from initial API query of device IDs
# client file to hold genericclient id and device IDs, colon separated
client_file="/tmp/mdm_client_list"
device_list="/tmp/mdm_device_list"

# Keeping 9 historical versions
log_name=~/fix_mdm_system_channel
echo "Logging to $log_name"

counter=9
while [ $counter -gt 0 ]
do
	file_number=$(( $counter - 1 ))

	if [ $file_number -eq 0 ]
	then
		if [ -f ${log_name}.log ]
		then
			mv ${log_name}.log ${log_name}-${counter}.log
		fi
	else
		if [ -f ${log_name}-${file_number}.log ]
		then
			mv ${log_name}-${file_number}.log ${log_name}-${counter}.log
		fi
	fi
	counter=$file_number
done

# Logging function
function echo_me {
        datef=$(date "+%Y-%m-%d %H:%M:%S.***")
        echo "${datef}|main|CUSTOM|CLIENT|fix_mdm_system_channel $1" >> ${log_name}.log
}

# Obtain response date for each MDM macOS device
function process_where_list {

mdm_command_date=$(/usr/local/filewave/postgresql/bin/psql mdm django  -t << EOF
SELECT response_date
FROM ios_commands
WHERE ($where_list)
AND request_type='DeviceInformation'
AND status=1
;
EOF
)
	# No date found, populate with old date.  Apple geek date alert!
	if [ -z $mdm_command_date ]
	then
		mdm_command_date=' 1984-01-24 12:00:00'
	fi
}

# For system and each user ID we want the respone_date
# Remember, latest user dates only required
function date_entry {

	# Pass to function to run psql query to pull response_date
	where_list="user_id=$user_id"
	process_where_list $where_list
	response_date=$mdm_command_date

	# Are we system or user
	# For user, build out an array so we can process them
	if [[ "$1" =~ "\|"$ ]]
	then
		system_date=${response_date%.*}
	else
		user_responses+=(${response_date%.*})
	fi

	echo_me "System date: $system_date"
	echo_me "Response array: ${user_responses[@]}"
}

# Process the list of users for a device
function process_user_list {

	# Example user_id_list:
	#  153 | username
	#  147 | 
	# System channel has no username

	while read line
	do
		user_id=$(echo $line | awk '{print $1}')
		user_name=$(echo $line | awk '{print $3}')

		if [ -z $user_name ]
		then
			echo_me "Processing System channel commands"
		else
			echo_me "Processing User commands"
		fi

		# Pass the user to a function to pull the date entries
		date_entry $line		

	done< <(echo $user_id_list)

	# Function will pass back an entry for system and an array for user dates
	# Newest user date only please
	newest_date=""
	last_date=""

	for a_date in ${user_responses[@]}
	do
		case $os_name in

			"Darwin")
				date_in_secs=$(date -j -f " %F %T" "${a_date}" +%s)
			;;
			"Linux")
				date_in_secs=$(date +%s --date "${a_date}")
			;;
		esac

		if [ -z $newest_date ]
		then
			last_date=$date_in_secs
			newest_date=${a_date}
		else
			if [ $date_in_secs -gt $last_date ]
			then
				last_date=$date_in_secs
				newest_date=${a_date}
			fi
		fi
	done
}

# Format response date for API patch command
function format_date {

	case $os_name in 

		"Darwin")
			date -j -f " %F %T" "$1" "+%FT%TZ"
		;;
		"Linux")
			date --date "$1" "+%FT%TZ"
		;;
	esac
}

# API to patch the response date values to Custom Fields
function patch_date_value {

	date_now=$(date +"%FT%TZ")

	echo_me "Sending API Patch command with the following..."
	echo_me "System_date $1"
	echo_me "User date $2"

	patch_data='
{
	"CustomFields":
		{
			"macos_mdm_system_date":
				{
					"exitCode":null,"status":0,"updateTime":"'$date_now'","value":"'$system_date'"
				},
			"macos_mdm_user_date":
				{
					"exitCode":null,"status":0,"updateTime":"'$date_now'","value":"'$newest_date'"
				}
		}
}'
	curl -X PATCH https://$server_dns:20445/inv/api/v1/client/$device_id_number -d $patch_data -H 'content-type: application/json' -k -s -H "authorization: $auth" 1>/dev/null
}

### BEGIN

os_name=$(uname -s)
where_list=""

# Each device will have multiple user iDs, one for system and possible multiple users
# Create an array of user IDs.  Will use a function to determine latest dates only and return just one users dates

# Inventory Query to get device_id for MDM enrolled macOS devices, configured for MDM Software Updates
query='
{
	"criteria":
		{"expressions":
			[
				{"column":"type","component":"OperatingSystem","operator":"is","qualifier":"OSX"},
				{"column":"su_via_mdm","component":"MacOSClient","operator":"=","qualifier":true},
				{"column":"is_mdm_enrolled_from_server_perspective","component":"MacOSClient","operator":"=","qualifier":true}
			],
			"logic":"all"
		},
	"fields":
		[
			{"column":"device_id","component":"Client"}
		],
		"main_component":"Client"
}'

read -r -d '' pyscript <<-"----EOF"
import json,sys;obj=json.load(sys.stdin)

for a_value in obj["values"]:
    print(a_value[0])
----EOF

device_serial_list=$(curl -s -k -H "Authorization: $auth"  https://${server_dns}:20443/inv/api/v1/query_result/ --data $query  -H "Content-Type: application/json" | /usr/local/filewave/python/bin/python -c "$pyscript")

# Populate the device list with device IDs
echo $device_serial_list > $device_list

# Convert device list into genericclient ID and device IDs
# Ensure file is empty:
printf "" > $client_file

while read line
do

device_id=$(/usr/local/filewave/postgresql/bin/psql mdm django  -t << EOF
SELECT id
FROM generic_genericclient
WHERE device_id='$line'
;
EOF
)

echo "${device_id}:${line}" >> $client_file

done< <(cat $device_list)

# Iterate the client file
while read device_line
do
	echo_me "---------------------------"
	echo_me "Processing Client ID: $device_line"
	declare -a user_responses=()

	# Reset the date stores for each new client
	system_date=""
	user_responses=()

user_id_list=$(/usr/local/filewave/postgresql/bin/psql mdm django  -t << EOF
SELECT id, ios_mdmuser.user
FROM ios_mdmuser
WHERE mdm_client_id=${device_line%%:*}
;
EOF
)
	# Function will pull the two required response dates: system and newest user
	process_user_list $user_id_list

	# Push these values to the Custom Fields.  Reformat date first:
	system_date=$(format_date $system_date) 
	newest_date=$(format_date $newest_date)

	# Use API to set date values for devices
	device_id_number=${device_line##*:}
	patch_date_value "$system_date" "$newest_date"

done< <(cat "$client_file")

exit 0
